#!/usr/bin/env python
# coding=utf-8

import parapheur     # Configuration
from parapheur.parapheur import pprint   # Colored printer
from parapheur.parapheur import config
import os
import re
import time

__version__ = "2017-03-28"
__config_section__ = "RecupArchives"

# Init REST API client
client = parapheur.getrestclient()

# Params
recup_folder = config.get(__config_section__, "folder")
# recup_folder = "/tmp/getdoc/"
page_size = config.get(__config_section__, "page_size")
# page_size = "500"

use_reduced_download_path = config.get(__config_section__, "use_reduced_download_path") == "true"
# use_reduced_download_path = False
purge = config.get(__config_section__, "purge") == "true"
# purge = False
download = config.get(__config_section__, "download") == "true"
# download = True

type_filter = config.get(__config_section__, "type_filter")
# type_filter = "*"
subtype_filter = config.get(__config_section__, "subtype_filter")
# subtype_filter = "*"
waiting_days = int(config.get(__config_section__, "waiting_days"))
# waiting_days = 0


# region Private methods


def move_incomplete_download_to_temp(path):
    temp_version = 1
    while os.path.exists("{0}_temp{1}".format(path, temp_version)):
        temp_version += 1
    os.rename("{0}".format(path), "{0}_temp{1}".format(path, temp_version))


def get_reduced_folder_path(dossier_type, dossier_subtype, dossier_title, dossier_id):
    """
    {rootFolder}/{Type}/{DossierName}/
    Possibly with a " (2)" suffix, if a folder with the same name already exists.

    This folder will be created with an empty Subtitle and NodeId file,
    to avoid any lost information.
    :return: download absolute path, or None if already exists
    """
    base_folder = "{0}/{1}".format(recup_folder, dossier_type)
    reduced_path = "{0}/{1}".format(base_folder, dossier_title)

    is_same_id = os.path.exists("{0}/.{1}".format(reduced_path, dossier_id))
    if is_same_id:
        return reduced_path

    if os.path.exists(reduced_path):
        version = 2

        while os.path.exists("{0} ({1})".format(reduced_path, version)):
            versioned_path = "{0} ({1})".format(reduced_path, version)

            is_same_id = os.path.exists("{0}/.{1}".format(versioned_path, dossier_id))
            if is_same_id:
                return versioned_path

            version += 1
        reduced_path = "{0} ({1})".format(reduced_path, version)

    os.makedirs(reduced_path, 0755)
    os.mknod("{0}/.{1}".format(reduced_path, dossier_id))
    os.mknod("{0}/.{1}".format(reduced_path, dossier_subtype))
    return reduced_path


def get_full_folder_path(dossier_type, dossier_subtype, dossier_title, dossier_id):
    """
    {rootFolder}/{Type}/{SubType}{DossierName}_{DossierId}/
    :return: download absolute path, or None if already exists
    """
    full_path = "{0}/{1}/{2}/{3}_{4}".format(recup_folder, dossier_type, dossier_subtype, dossier_title, dossier_id)

    if not os.path.exists(full_path):
        os.makedirs(full_path, 0755)

    return full_path


def cleanup_special_chars(string):

    # Windows Forbidden punctuation
    cleaned = re.sub(u"<", "(", string)
    cleaned = re.sub(u">", ")", cleaned)
    cleaned = re.sub(u":", "=", cleaned)
    cleaned = re.sub(u"\"", "''", cleaned)
    cleaned = re.sub(u"[\\/\|]", "-", cleaned)
    cleaned = re.sub(u"[\*\?%€&£$§]", "_", cleaned)

    # Special chars
    cleaned = re.sub(u'[ÀÁÂÄ]', 'A', cleaned)
    cleaned = re.sub(u'[ÈÉÊË]', 'E', cleaned)
    cleaned = re.sub(u'[ÍÌÎÏ]', 'I', cleaned)
    cleaned = re.sub(u'[ÒÓÔÖ]', 'O', cleaned)
    cleaned = re.sub(u'[ÙÚÛÜ]', 'U', cleaned)
    cleaned = re.sub(u'[áàâä]', 'a', cleaned)
    cleaned = re.sub(u'[éèêë]', 'e', cleaned)
    cleaned = re.sub(u'[íìîï]', 'i', cleaned)
    cleaned = re.sub(u'[óòôö]', 'o', cleaned)
    cleaned = re.sub(u'[úùûü]', 'u', cleaned)
    cleaned = re.sub(u'Æ', 'AE', cleaned)
    cleaned = re.sub(u'æ', 'ae', cleaned)
    cleaned = re.sub(u'Œ', 'OE', cleaned)
    cleaned = re.sub(u'œ', 'oe', cleaned)
    cleaned = re.sub(u'Ç', 'c', cleaned)
    cleaned = re.sub(u'ç', 'c', cleaned)

    if len(cleaned) == 0:
        cleaned = "dossier_sans_nom"

    return cleaned


# endregion


type_filter = type_filter.replace(" ", "%20")
subtype_filter = subtype_filter.replace(" ", "%20")
download_folder_path = None

if client.islogged:

    # Fetch folders

    page = 0
    dossiers_archive = []

    dossiers_fetched = [1]
    while len(dossiers_fetched) > 0:

        dossiers_fetched = client.doget(
            "/parapheur/archives",
            dict(
                page=str(page),
                filter='{"and":[{"or":[{"ph:typeMetier":"%s"}]},{"or":[{"ph:soustypeMetier":"%s"}]}]}' % (type_filter, subtype_filter),
                pageSize=page_size
            )
        )

        dossiers_archive += dossiers_fetched
        pprint.log("Page {0} : {1} dossiers".format(page, len(dossiers_fetched)))
        page += 1

    pprint.log("{0} dossier(s) trouvé(s)".format(len(dossiers_archive)))

    # Waiting delay

    if waiting_days > 0:
        timestamp = (int(time.time()) - 60 * 60 * 24 * waiting_days) * 1000
        dossiers_archive = [d for d in dossiers_archive if d['created'] < timestamp]
        pprint.log("{0} dossier(s) apres le delai de carence".format(len(dossiers_archive)))

    # Download

    for dossier_index in range(0, len(dossiers_archive)):

        dossier = dossiers_archive[dossier_index]
        title_clean = cleanup_special_chars(dossier['title'])
        type_clean = cleanup_special_chars(dossier['type'])
        subtype_clean = cleanup_special_chars(dossier['sousType'])

        if download:

            # Create folders

            if use_reduced_download_path:
                download_folder_path = get_reduced_folder_path(type_clean, subtype_clean, title_clean, dossier['id'])
            else:
                download_folder_path = get_full_folder_path(type_clean, subtype_clean, title_clean, dossier['id'])

            # Cleanup

            is_already_downloaded = os.path.exists("{0}/.done".format(download_folder_path))

            if use_reduced_download_path:
                folder_already_contains_data = len(os.listdir(download_folder_path)) > 2
            else:
                folder_already_contains_data = len(os.listdir(download_folder_path)) > 0

            if folder_already_contains_data and not is_already_downloaded:
                move_incomplete_download_to_temp(download_folder_path)

                if use_reduced_download_path:
                    download_folder_path = get_reduced_folder_path(type_clean, subtype_clean, title_clean, dossier['id'])
                else:
                    download_folder_path = get_full_folder_path(type_clean, subtype_clean, title_clean, dossier['id'])

            # Download content

            if not is_already_downloaded:

                content_url = "/api/node/content/workspace/SpacesStore/{0}/{1}".format(dossier['id'], title_clean)
                content_url = content_url.replace(" ", "%20")
                client.dodownload(content_url, "{0}/{1}".format(download_folder_path, title_clean))

                if dossier['original'] == "true":

                    if dossier['isXemEnabled']:
                        dossier_distant_original_name = title_clean
                        dossier_local_original_name = "{0}_original.xml".format(title_clean)
                    elif hasattr(dossier, 'originalName') and dossier['originalName'] is not None:
                        dossier_distant_original_name = cleanup_special_chars(dossier['originalName'])
                        dossier_local_original_name = "original_" + dossier_distant_original_name
                    else:
                        dossier_distant_original_name = title_clean
                        dossier_local_original_name = "original_" + title_clean

                    original_url = "/api/node/content%3bph%3aoriginal/workspace/SpacesStore/{0}/{1}".format(dossier['id'], dossier_distant_original_name)
                    original_url = original_url.replace(" ", "%20")
                    client.dodownload(original_url, "{0}/{1}".format(download_folder_path, dossier_local_original_name))

                if dossier['sig'] == "true":
                    sign_url = "/api/node/content%3bph%3asig/workspace/SpacesStore/{0}/{1}_sig.zip".format(dossier['id'], title_clean)
                    sign_url = sign_url.replace(" ", "%20")
                    client.dodownload(sign_url, "{0}/{1}_sig.zip".format(download_folder_path, title_clean))

                os.mknod("{0}/.done".format(download_folder_path))
                pprint.success("Downloaded : {0} ({1}/{2})".format(dossier['id'], dossier_index + 1, len(dossiers_archive)))
            else:
                pprint.warning("Already downloaded : {0} ({1}/{2})".format(dossier['id'], dossier_index + 1, len(dossiers_archive)))

        if purge:
            if download:
                if os.path.exists("{0}/.done".format(download_folder_path)):
                    client.executescript("removeNode.js", format=(dossier['id'],))
            else:
                client.executescript("removeNode.js", format=(dossier['id'],))
                pprint.success("Deleted : {0} ({1}/{2})".format(dossier['id'], dossier_index + 1, len(dossiers_archive)))


pprint.success("Done", True)



#!/usr/bin/env python
# coding=utf-8

"""
CeCILL Copyright (c) 2016-2017, Libriciel SCOP
Initiated and by Libriciel SCOP

contact@libriciel.coop

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
"""

from parapheur.parapheur import pprint  # Colored printer
import io
import os
import subprocess
import socket
import sys
import platform
import multiprocessing
import requests
# import MySQLdb
import pymysql.cursors
from pymysql.constants import ER
# from packaging import version
from pkg_resources import parse_version

req_version = (3, 0)
cur_version = sys.version_info
isp3 = cur_version >= req_version
# pprint.log(cur_version)

if isp3:
    from io import StringIO
    # noinspection PyCompatibility
    import configparser as ConfigParser
else:
    # noinspection PyCompatibility
    from StringIO import StringIO
    # noinspection PyCompatibility
    import ConfigParser

__author__ = 'Stephane Vast'
__version__ = '1.0.1'

defaut_install_depot = "/opt/_install"
defaut_iparapheur_root = "/opt/iParapheur"
mysqluser = "alf"
mysqlpwd = ""
mysqlbase = ""


def isexistsdirectory(repertoire):
    pprint.header("#", False, ' ')
    pprint.info("Répertoire", False, ' ')
    pprint.info(repertoire.ljust(35), True, ' ')
    if os.path.exists(repertoire):
        pprint.success('{:>10s}'.format("OK"), True)
        return True
    else:
        pprint.warning('{:>10s}'.format("absent"))
        return False


def isexistssubdir(repertoire, sousrep):
    pprint.header("#", False, ' ')
    pprint.info("  subdir", False, ' ')
    pprint.info(sousrep.ljust(37), True, ' ')
    if os.path.exists("{0}/{1}".format(repertoire, sousrep)):
        pprint.success('{:>10s}'.format("OK"), True)
        return True
    else:
        pprint.warning('{:>10s}'.format("absent"))
        return False


def isexistsfile(repertoire, fichier):
    pprint.header("#", False, ' ')
    pprint.info(" fichier", False, ' ')
    pprint.info(fichier.ljust(37), True, ' ')
    if os.path.exists("{0}/{1}".format(repertoire, fichier)):
        pprint.success('{:>10s}'.format("OK"), True)
        return True
    else:
        pprint.warning('{:>10s}'.format("absent"))
        return False


def which(program):
    import os
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None


def showtheheader():
    pprint.header('{:30s}{:>30s}'.format('Tests opérés pour i-Parapheur', 'Resultat'))
    pprint.header('=' * 60)
    pprint.header("# Check list pour", False, ' ')
    pprint.log("i-Parapheur", True, ' ')
    pprint.header("       résultats:", False, ' ')
    pprint.success("OK", True, ' ')
    pprint.warning("warn", True, ' ')
    pprint.error("Fail", True)
    pprint.header("# ")


# TESTS sur PRE-REQUIS: hardware (CPU  , RAM )
'''     Hardware    : nb CPU  , RAM , taille Disque ?
		OS Linux    : nom + version / NeSaitPas
		MySQL       : version? .... / NeSaitPas , local/déporté
		NginX       : version? .... / NeSaitPas , local/déporté
		JDK serveur : version? .... / NeSaitPas
		LibreOffice : version? .... / NeSaitPas
		GhostScript : version? .... / NeSaitPas
	Pour i-Parapheur, sécurité:
		Fournisseur certificats HTTPS (web,WS): ... , ... / NeSaitPas
		Date d'expiration cert. HTTPS (web,WS): ... , ... / NeSaitPas
		Version LiberSign : .....  / NeSaitPas	'''


# nb CPU - BASH: grep -c "processor" /proc/cpuinfo
def check_hardware():
    pprint.header("#", False, ' ')
    pprint.log("---- Check pre-requis systeme ----", True)

    pprint.header("#", False, ' ')
    pprint.info("Nombre de CPU disponibles (minimum 4)".ljust(46), False, ' ')
    nbcpu = multiprocessing.cpu_count()
    if nbcpu >= 4:
        pprint.success('{:>10d}'.format(nbcpu), True)
    elif nbcpu >= 2:
        pprint.warning('{:>10d}'.format(nbcpu))
    else:
        pprint.error('{:>10d}'.format(nbcpu))

    pprint.header("#", False, ' ')
    pprint.info("Taille Memoire totale (minimum 5 Go)".ljust(46), False, ' ')
    mem_bytes = os.sysconf('SC_PAGE_SIZE') * os.sysconf('SC_PHYS_PAGES')  # e.g. 4015976448
    mem_gib = mem_bytes / (1024. ** 3)  # e.g. 3.74
    if mem_gib >= 4.5:
        pprint.success('{:>7.2f} Go'.format(mem_gib), True)
    elif mem_gib > 3.75:
        pprint.warning('{:>7.2f} Go'.format(mem_gib))
    else:
        pprint.error('{:>7.2f} Go'.format(mem_gib))

    pprint.header("#", False, ' ')
    pprint.info("Plateforme {0} : architecture {1}".format(os.uname()[0], os.uname()[4]).ljust(46), False, ' ')
    l_arch = platform.architecture()[0]
    if l_arch == "64bit":
        pprint.success('{:>10s}'.format(l_arch), True)
    else:
        pprint.error('{:>10s}'.format(l_arch), True)
        pprint.error("Erreur: Le systeme doit etre 64bit pour recevoir i-Parapheur. STOP", True)
        sys.exit()
    #	pprint.log(platform.platform())
    #	pprint.log(platform.release())
    #	pprint.log(platform.system()) # Linux
    #	pprint.log(platform.version())
    pprint.header("#", False, ' ')
    # pprint.log(platform.linux_distribution())  # ('Ubuntu', '16.04', 'xenial')
    pprint.info("Distribution: {0} {1} ({2})".format(platform.linux_distribution()[0],
                                                     platform.linux_distribution()[1],
                                                     platform.linux_distribution()[2]).ljust(46), False, ' ')
    if platform.linux_distribution()[0] == 'Ubuntu':
        if platform.linux_distribution()[1] == '16.04':
            pprint.success('{:>10s}'.format("OK"), True)
        elif platform.linux_distribution()[1] == '14.04':
            pprint.warning('{:>10s}'.format("OK"), True)
        elif platform.linux_distribution()[1] == '12.04':
            pprint.warning('{:>10s}'.format("migrer"), True)
        else:
            pprint.error('{:>10s}'.format("A qualifier"), True)
    elif platform.linux_distribution()[0] == 'debian':
        if platform.linux_distribution()[1].startswith('8'):
            pprint.success('{:>10s}'.format("OK"), True)
        elif platform.linux_distribution()[1].startswith('7'):
            pprint.warning('{:>10s}'.format("OK"), True)
        else:
            pprint.error('{:>10s}'.format("non conforme"), True)
    else:  # ajouter CentOS 6/7 , RHEL: 'SuSE', 'redhat', 'mandrake'
        pprint.error('{:>10s}'.format("inconnu"), True)

    pprint.header("#", False, ' ')
    pprint.info("swappiness (valeur <=10)".ljust(46), False, ' ')
    PROCFS_PATH = "/proc/sys/vm/swappiness"
    if os.path.isfile(PROCFS_PATH) and os.access(PROCFS_PATH, os.R_OK):
        myfile = open(PROCFS_PATH, 'r')
        for line in myfile:
            swappiness = int(line.rstrip("\n"))
            if swappiness > 10:
                pprint.error('{:>10d}'.format(swappiness), True)
            else:
                pprint.success('{:>10d}'.format(swappiness), True)
        myfile.close()


# pprint.log(os.getlogin())
# pprint.log(os.uname())


def check_server_socket(address, port):
    # Create a TCP socket
    s = socket.socket()
    # print "Attempting to connect to %s on port %s" % (address, port)
    try:
        s.connect((address, port))
        # print "Connected to %s on port %s" % (address, port)
        s.close()
        return True
    except socket.error as e:
        print("Connection to %s on port %s failed: %s" % (address, port, e))
        return False


def issitereachable(theurl):
    pprint.header("#", False, ' ')
    pprint.info("Test site {0}".format(theurl).ljust(46), False, ' ')

    response = requests.get(theurl)

    if not response.ok:
        pprint.error('{:>10s}'.format("Erreur"), True)
        pprint.error("Erreur lors de la requête {0}: Code d'erreur {1}".
                     format(theurl, response.status_code),
                     True)
        pprint.error(response.getvalue())
    else:
        pprint.success('{:>10s}'.format("OK"), True)


# besoin HTTP/HTTPS sortant, accès http://crl.adullact.org http://libersign.adullact-projet.fr
def check_network_needed():
    pprint.header("#", False, ' ')
    pprint.log("---- Check pre-requis reseau ----", True)
    issitereachable("http://crl.adullact.org")
    issitereachable("http://libersign.adullact-projet.fr")


# pprint.error('{:>10s}'.format("non fait"), True)


def check_mandatory_command(thecommand):
    pprint.header("#", False, ' ')
    pprint.info("Commande : {0}".format(thecommand).ljust(46), False, ' ')
    if which(thecommand):
        pprint.success('{:>10s}'.format("OK"))
    else:
        pprint.error('{:>10s}'.format("Absent"), True)


def check_required_software():
    pprint.header("#", False, ' ')
    pprint.log("---- Check pre-requis logiciels selon manuel ----", True)

    if isexistsdirectory(defaut_install_depot):
        isexistssubdir(defaut_install_depot, "confs")
    if not isexistsdirectory(defaut_iparapheur_root):
        pprint.error("Erreur: Le répertoire {0} doit être présent. STOP".format(defaut_iparapheur_root), True)
        sys.exit()

    da_commands = ['at', 'tar', 'crontab', 'unzip', 'mailx',
                   'mysql', 'mysqldump', 'mysqloptimize', '/opt/java/jdk1.6.0_45/bin/java']
    for to_test in da_commands:
        check_mandatory_command(to_test)
    # check_mandatory_command("FauxPositif")


def check_smtp_needed(smtp_srv):
    pprint.header("#", False, ' ')
    pprint.log("---- Check SMTP ----", True)
    if check_server_socket(smtp_srv, 25):
        pprint.info("un service SMTP est présent sur {0}".format(smtp_srv).ljust(46), False, ' ')
        pprint.success("ok")
    else:
        pprint.warning("ko")
    pprint.warning('{:>10s}'.format("TODO"), True)


def check_https_service_config():
    pprint.header("#", False, ' ')
    pprint.log("---- Check configuration service HTTPS basique ----", True)

    if not isexistsdirectory("/etc/nginx"):
        pprint.header("#   ", False, ' ')
        pprint.warning("Pas de configuration NginX, c'est pourtant le serveur HTTPS à utiliser", True)
        return False
    isexistsdirectory("/etc/nginx/conf.d")
    if isexistsdirectory("/etc/nginx/ssl"):
        isexistsfile("/etc/nginx/ssl", "recup_crl_nginx.sh")
        isexistssubdir("/etc/nginx/ssl", "validca")

    # Vérifier process Nginx pas trop vieux (p/r lancement crontab) ?
    line = subprocess.check_output('nginx -v', stderr=subprocess.STDOUT, shell=True).decode("utf-8")
    outl, outr = line.rstrip("\n").split("nginx/")
    pprint.header("#", False, ' ')
    pprint.info("Version detectee NginX:     {0}".format(outr.rstrip()).ljust(46), False, ' ')
    # if version.parse("1.8.0") < version.parse(outr.rstrip()):
    if parse_version("1.8.0") < parse_version(outr.rstrip()):
        pprint.success('{:>10s}'.format(">1.8.0, OK"), True)
    else:
        pprint.error('{:>10s}'.format("< 1.8.0"), True)

    nginxserver = "localhost"
    nginxport = 443
    pprint.header("#", False, ' ')
    pprint.info("Service NginX sur {0}:{1}".format(nginxserver, nginxport).ljust(46), False, ' ')
    if not check_server_socket(nginxserver, nginxport):
        pprint.warning('{:>10s}'.format("inactif"), True)
    else:
        pprint.success('{:>10s}'.format("actif"), True)

    print("   CAVEAT: tests sur certificat valide, validca OK, etc.")


## OpenSSL: récupérer la chaîne de certificats SSL d’un host
## https://blog.hbis.fr/2017/02/11/openssl-certificate_chain/
# echo | openssl s_client -connect iparapheurfl.demonstrations.libriciel.fr:443 -showcerts 2>&1 | sed -ne '/-BEGIN CERTIFICATE-/,/-END CERTIFICATE-/p' > mycert.pem




def check_mysql_service_config(varconf):
    pprint.header("#", False, ' ')
    pprint.log("---- Check configuration service MySQL ----  TODO", True)

    mysqlserver = "localhost"
    mysqlport = 3306

    pprint.header("#", False, ' ')
    pprint.info("Service MySQL sur {0}:{1}".format(mysqlserver, mysqlport).ljust(46), False, ' ')
    if not check_server_socket(mysqlserver, mysqlport):
        pprint.warning('{:>10s}'.format("inactif"), True)
        return False
    else:
        pprint.success('{:>10s}'.format("actif"), True)

        mysqluser = varconf.get("Parapheur", "db.username")
        mysqlpwd = varconf.get("Parapheur", "db.password")
        mysqlbase = varconf.get("Parapheur", "db.name")
        pprint.header("#", False, ' ')
        pprint.info("DB '{3}' sur {0}@{1}:{2}".format(mysqluser, mysqlserver, mysqlport, mysqlbase).ljust(46), False,
                    ' ')

        try:
            cur_max_cnx = ""
            parametre_mysql = ""

            cnx = pymysql.connect(user=mysqluser, password=mysqlpwd,
                                          host=mysqlserver, database=mysqlbase)
            pprint.success('{:>10s}'.format("OK"), True)

            cursor = cnx.cursor()
            query = "SELECT @@GLOBAL.max_connections as res;"
            cursor.execute(query)
            for res in cursor:
                cur_max_cnx = res[0]
            pprint.header("#", False, ' ')
            pprint.info('Nombre de connexions maxi      = {:>6d}'.format(cur_max_cnx).ljust(46), False, ' ')
            if cur_max_cnx < 360:
                pprint.error('{:>10s}'.format("< 360"), True)
            else:
                pprint.success('{:>10s}'.format(">=360, OK"), True)

            query = "SELECT @@GLOBAL.innodb_file_per_table as res;"
            cursor.execute(query)
            for res in cursor:
                parametre_mysql = res[0]
            pprint.header("#", False, ' ')
            pprint.info('innodb_file_per_table          = {0:>6d}'.format(parametre_mysql).ljust(46), False, ' ')
            if parametre_mysql != 1:
                pprint.error('{0:>10s}'.format("!= 1"), True)
            else:
                pprint.success('{0:>10s}'.format("=1, OK"), True)

            query = "SELECT @@GLOBAL.open_files_limit as res;"
            cursor.execute(query)
            for res in cursor:
                parametre_mysql = res[0]
            pprint.header("#", False, ' ')
            pprint.info('open_files_limit               = {0:>6d}'.format(parametre_mysql).ljust(46), False, ' ')
            if parametre_mysql < 8192:
                pprint.error('{0:>10s}'.format("< 8192"), True)
            else:
                pprint.success('{0:>10s}'.format(" OK"), True)

            query = "SELECT @@GLOBAL.wait_timeout as res;"
            cursor.execute(query)
            for res in cursor:
                parametre_mysql = res[0]
            pprint.header("#", False, ' ')
            pprint.info('wait_timeout                   = {0:>6d}'.format(parametre_mysql).ljust(46), False, ' ')
            if parametre_mysql < 28800:
                pprint.error('{0:>10s}'.format("< 28800"), True)
            else:
                pprint.success('{0:>10s}'.format("  OK"), True)

            query = "SELECT @@GLOBAL.innodb_locks_unsafe_for_binlog as res;"
            cursor.execute(query)
            for res in cursor:
                parametre_mysql = res[0]
            pprint.header("#", False, ' ')
            pprint.info('innodb_locks_unsafe_for_binlog = {0:>6d}'.format(parametre_mysql).ljust(46), False, ' ')
            if parametre_mysql != 1:
                pprint.error('{0:>10s}'.format("!= 1"), True)
            else:
                pprint.success('{0:>10s}'.format("=1, OK"), True)

            cursor.close()

        except pymysql.Error as err:
            pprint.error('{0:>10s}'.format("Erreur"), True)
            if err.errno == ER.ACCESS_DENIED_ERROR:
                print("Something is wrong with your user name or password")
            elif err.errno == ER.BAD_DB_ERROR:
                print("Database does not exist")
            else:
                print(err)
        else:
            cnx.close()
        pprint.warning('{0:>10s}'.format("TODO"), True)


def check_isexists_alfrescoglobal():
    pprint.header("#", False, ' ')
    pprint.log("---- Exists alfresco-global.properties  ? ----  ", True)

    # Alfresco-global.properties
    isexistsdirectory(defaut_iparapheur_root)
    isexistssubdir(defaut_iparapheur_root, "tomcat/shared/classes")
    return isexistsfile("{0}/tomcat/shared/classes".format(defaut_iparapheur_root), "alfresco-global.properties")


def check_config_alfrescoglobal(varconf):
    pprint.header("#", False, ' ')
    pprint.log("---- Check alfresco-global.properties ----  TODO", True)

    alf_dir_root = varconf.get("Parapheur", "dir.root")
    # pprint.info(varconf.options("Parapheur"))
    # pprint.info(varconf.items("Parapheur"))

    pprint.header("#", False, ' ')
    pprint.info("   alf_dir_root = {0}".format(alf_dir_root))


def check_config_ghostscript():
    pprint.header("#", False, ' ')
    pprint.log("---- Check config GhostScript ----", True)
    libgs_path = defaut_iparapheur_root + "/common/lib/libgs.so"
    if not isexistsfile("{0}/common/lib".format(defaut_iparapheur_root), "libgs.so"):
        return
    libgsstat = os.stat(libgs_path)
    pprint.header("#", False, ' ')
    pprint.info("Taille libgs.so = {0} octets".format(libgsstat.st_size).ljust(46), False, ' ')
    if libgsstat.st_size < 19 * 1024 * 1024:
        pprint.error('{0:>10s}'.format("< 19Mo"), True)
    else:
        pprint.success('{0:>10s}'.format("> 19Mo, OK"), True)
    command_line = "ldconfig -n /opt/iParapheur/common/lib/ -v | grep libgs.so"
    output = subprocess.check_output(command_line, shell=True)
    outl, outr = output.split("-> ")
    pprint.header("#", False, ' ')
    pprint.info("Version detectee: {0}".format(outr.rstrip()), False)


def check_files_needed():
    pprint.header("#", False, ' ')
    pprint.log("---- Check présence fichiers post-config ----", True)
    da_files = ['backup_parapheur.sh', 'custom-wsdl.sh',
                'deployWarIparapheur.sh', 'iparaph-updateAMP.sh',
                'logrotate-iparapheur.conf', 'nettoieEntrepot.sh', 'nettoieLogs.sh',
                'purge-xemwebview.sh', 'srgb.profile', 'verdanai.ttf',
                'warn_needPurge.sh']
    for to_test in da_files:
        isexistsfile("/opt/iParapheur", to_test)


# chapitre  XEMELIOS
def check_xemwebview_service_config():
    pprint.header("#", False, ' ')
    pprint.log("---- Check configuration service Xemwebviewer ----  TODO", True)
    isexistsfile("/etc/init.d", "xemwebview")
    if not isexistsdirectory("/var/tmp/bl-xemwebviewer"):
        pprint.header("#   ", False, ' ')
        pprint.warning("Le service Xemelios ne fonctionnera pas: manquent les répertoires temporaires.")
    else:
        da_reps = ['xwv-cache', 'xwv-extract', 'xwv-shared']
        for to_test in da_reps:
            if not isexistssubdir("/var/tmp/bl-xemwebviewer", to_test):
                pprint.header("#   ", False, ' ')
                pprint.warning("Le service Xemelios ne fonctionnera pas: manque le répertoire {0}.".format(to_test))


################################################################
################################################################
showtheheader()
check_hardware()
check_network_needed()
check_required_software()
check_smtp_needed("localhost")

check_https_service_config()

if not check_isexists_alfrescoglobal():
    pprint.error("BAD")
    sys.exit()

ALF_CONFIG_PATH = "{0}/tomcat/shared/classes/alfresco-global.properties".format(defaut_iparapheur_root)
'''
def get_config_app(varfichier):
	### lire https://stackoverflow.com/questions/2819696/parsing-properties-file-in-python/25493615#25493615
	####   https://stackoverflow.com/questions/2885190/using-pythons-configparser-to-read-a-file-without-section-name
	with open(varfichier, 'r') as f:
	    config_string = '[Parapheur]\n' + f.read()
	# config_fp = StringIO.StringIO(config_string)
	config_fp = io.BytesIO(config_string)
	config = ConfigParser.RawConfigParser()
	return config.readfp(config_fp)
'''
with open(ALF_CONFIG_PATH, 'r') as f:
    config_string = '[Parapheur]\n' + f.read()
config_fp = io.BytesIO(config_string)
config = ConfigParser.RawConfigParser()
config.readfp(config_fp)
# alfrescoconfig = get_config_app(CONFIG_PATH)
# print(config.items("Parapheur"))
# print(config.get("Parapheur", "dir.root"))

check_config_alfrescoglobal(config)
check_mysql_service_config(config)

# check  "libgs"
check_config_ghostscript()

# tests présence verdanai.ttf , srgb.profile ,etc.
check_files_needed()

# ulimit dans alfresco.sh


check_xemwebview_service_config()

pprint.info(".end.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Ce module est un test de client parapheur
"""
__version__ = "0.1.0.4826"


def getrestclient():
    # We have to define it here, because we use it everywhere in scripts
    import parapheur
    return parapheur.getrestclient()


def getsoapclient(user=None, password=None):
    import parapheur
    return parapheur.getsoapclient(user, password)


# coding=utf-8
# module Parapheur
"""
CeCILL Copyright (c) 2006-2015, ADULLACT-projet
Initiated by ADULLACT-projet S.A.
Developped by ADULLACT-projet S.A.

contact@adullact-projet.coop

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
"""
import base64
import os
import requests
import subprocess
import sys
from requests.exceptions import SSLError

import magic
from suds.cache import NoCache
from suds.client import Client as Sudsclient
from suds.plugin import MessagePlugin, DocumentPlugin
from suds.transport import Reply
from suds.transport.https import HttpAuthenticated

import pprint

req_version = (3, 0)
cur_version = sys.version_info

if cur_version >= req_version:
    from io import StringIO
else:
    # noinspection PyCompatibility
    from StringIO import StringIO

__author__ = 'lhameury'


class RequestsTransport(HttpAuthenticated):
    def __init__(self, **kwargs):
        self.cert = kwargs.pop('cert', None)
        self.username = kwargs.pop('username', None)
        self.password = kwargs.pop('password', None)
        # super won't work because not using new style class
        HttpAuthenticated.__init__(self, **kwargs)

    def open(self, request):
        """
        Fetches the WSDL using cert.
        :param request: The request object
        """
        self.addcredentials(request)
        if "https" in request.url:
            resp = requests.get(request.url, data=request.message,
                                headers=request.headers, cert=self.cert, verify=True,
                                auth=(self.username, self.password))
        else:
            resp = requests.get(request.url, data=request.message,
                                headers=request.headers)
        result = StringIO(resp.content)
        return result

    def send(self, request):
        # Dirty hack... can not be handled in filter ! BEGIN
        request.message = request.message.replace("contentType", "xm:contentType")
        # END
        self.addcredentials(request)
        resp = requests.post(request.url, data=request.message,
                             headers=request.headers, cert=self.cert, verify=True,
                             auth=(self.username, self.password))
        result = Reply(resp.status_code, resp.headers, resp.content)
        return result


class Filter(MessagePlugin):
    def __init__(self):
        pass

    def marshalled(self, context):
        context.envelope.set('xmlns:xm', 'http://www.w3.org/2005/05/xmlmime')

    def received(self, context):
        reply = context.reply
        context.reply = reply[reply.find("<?xml version"):reply.rfind(">") + 1]


class Handlewsdl(DocumentPlugin):
    def __init__(self):
        pass

    def loaded(self, context):
        # Dirty hack ! Le type DossierID bloque !
        context.document = context.document.replace('type="iph:DossierID"', 'type="xsd:string"')


class Webservice:
    def __init__(self, config, user=None, password=None):
        pprint.log("\n[Parapheur-SOAP]", True)
        # Nom d'utilistaeur
        self.username = user if user is not None else config.get("Parapheur", "username")
        # Mot de passe utilisateur
        self.__password = password if password is not None else config.get("Parapheur", "password")
        # URL webservice
        self.__url = "https://" + config.get("Parapheur", "server") + "/ws-iparapheur"
        # Hostname
        self.__hostname = config.get("Parapheur", "server")
        pprint.info("Serveur     : " + self.__url)
        # Certificat webservice
        self.__cert = (
            os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', "files/public.pem"),
            os.path.join(os.path.abspath(os.path.dirname(__file__)), '..', "files/private.pem"))

        # On vérifie que l'AC est fournie
        autoritypath = os.path.join('/tmp', 'autority.pem')
        # Si non, on récupère la chaîne via une commande openssl
        if not os.path.isfile(autoritypath):
            command = "echo \"\" | " \
                      "openssl s_client -showcerts -connect " + self.__hostname + ":443 2>/dev/null | " \
                                                                                  "sed -n -e '/BEGIN\\ CERTIFICATE/,/END\\ CERTIFICATE/ p'"
            p = subprocess.Popen(command,
                                 shell=True, stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
            aut = p.stdout.read()
            with open(autoritypath, "w") as text_file:
                text_file.write(aut)

        os.environ['REQUESTS_CA_BUNDLE'] = autoritypath

        pprint.info("Utilisateur : " + self.username)

        if os.path.isfile(self.__cert[0]):
            credentials = dict(username=self.username,
                               password=self.__password,
                               cert=self.__cert)
            t = RequestsTransport(**credentials)
            try:
                self.api = Sudsclient(self.__url + '?wsdl', plugins=[Handlewsdl(), Filter()],
                                      location=self.__url, transport=t, cache=NoCache())
                self.api.service.echo("Coucou, ici Python !")
                pprint.success("OK", True)
            except SSLError as e:
                print(e)
        else:
            pprint.error("Fichier {0} introuvable".format(self.__cert), True)

    def call(self):
        return self.api.service

    def listmethods(self):
        return self.api

    def create(self, objectname):
        return self.api.factory.create(objectname)

    @staticmethod
    def loaddocument(chunk_size, location):
        base64file = ""
        with open(location, 'rb') as f:
            while True:
                data = f.read(chunk_size)
                if data:
                    base64file += base64.b64encode(data)
                else:
                    break
        return base64file

    def loadsig(self, location):
        loadedfile = self.create("TypeDoc")
        loadedfile["value"] = self.loaddocument(8192, location)
        loadedfile["_contentType"] = "application/pkcs7-signature"
        return loadedfile

    def loadfile(self, location):
        loadedfile = self.create("TypeDoc")
        loadedfile["value"] = self.loaddocument(8192, location)
        mime = magic.Magic(mime=True)
        loadedfile["_contentType"] = mime.from_file(location)
        return loadedfile

    def loadannexefile(self, location, name, mimetype="UTF-8"):
        annexes = self.create("TypeDocAnnexes")
        annexe = self.create("DocAnnexe")
        annexe["nom"] = name
        annexe["fichier"] = self.loadfile(location)
        mime = magic.Magic(mime=True)
        annexe["mimetype"] = mime.from_file(location)
        annexe["encoding"] = mimetype
        annexe["signature"] = None
        annexes["DocAnnexe"] = [annexe]
        return annexes



# coding=utf-8
# module Parapheur
"""
CeCILL Copyright (c) 2006-2015, ADULLACT-projet
Initiated by ADULLACT-projet S.A.
Developped by ADULLACT-projet S.A.

contact@adullact-projet.coop

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
"""
from __future__ import print_function

__author__ = 'lhameury'


def __init__():
    pass


def cstr(v):
    if isinstance(v, str):
        try:
            return v.encode('utf-8')
        except UnicodeDecodeError:
            return v
    return str(v)


def log(tolog, bold=False, end='\n'):
    print(("", __BOLD__)[bold] + cstr(tolog) + __ENDC__, end=end)


def header(tolog, bold=False, end='\n'):
    print(__HEADER__ + ("", __BOLD__)[bold] + cstr(tolog) + __ENDC__, end=end)


def info(tolog, bold=False, end='\n'):
    print(__OKBLUE__ + ("", __BOLD__)[bold] + cstr(tolog) + __ENDC__, end=end)


def success(tolog, bold=False, end='\n'):
    print(__OKGREEN__ + ("", __BOLD__)[bold] + cstr(tolog) + __ENDC__, end=end)


def warning(tolog, bold=False, end='\n'):
    print(__WARNING__ + ("", __BOLD__)[bold] + cstr(tolog) + __ENDC__, end=end)


def error(tolog, bold=True, end='\n'):
    print(__FAIL__ + ("", __BOLD__)[bold] + cstr(tolog) + __ENDC__, end=end)


__HEADER__ = '\033[95m'
__OKBLUE__ = '\033[94m'
__OKGREEN__ = '\033[92m'
__WARNING__ = '\033[93m'
__FAIL__ = '\033[91m'
__ENDC__ = '\033[0m'
__BOLD__ = '\033[1m'
__UNDERLINE__ = '\033[4m'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import parapheur

setup(

    name='iparapheur-utils',

    version=parapheur.__version__,

    # Liste les packages à insérer dans la distribution
    # plutôt que de le faire à la main, on utilise la foncton
    # find_packages() de setuptools qui va cherche tous les packages
    # python recursivement dans le dossier courant.
    packages=find_packages(),

    author="Lukas Hameury",

    # Votre email, sachant qu'il sera publique visible, avec tous les risques
    # que ça implique.
    author_email="lukas.hameury@libriciel.fr",

    # Une description courte
    description="Client python pour i-Parapheur",

    # Une description longue, sera affichée pour présenter la lib
    # Généralement on dump le README ici
    long_description=open('README.md').read(),

    # Vous pouvez rajouter une liste de dépendances pour votre lib
    # et même préciser une version. A l'installation, Python essayera de
    # les télécharger et les installer.
    #
    # Ex: ["gunicorn", "docutils >= 0.3", "lxml==0.5a7"]
    #
    # Dans notre cas on en a pas besoin, donc je le commente, mais je le
    # laisse pour que vous sachiez que ça existe car c'est très utile.
    install_requires=[
        'python-magic',
        'suds',
        'requests>=2.13.0',
        'importlib',
        'PyMySql'
    ],

    # Active la prise en compte du fichier MANIFEST.in
    include_package_data=True,

    # Une url qui pointe vers la page officielle de votre lib
    url='https://gitlab.libriciel.fr/i-parapheur/client-python',

    # classifiers=[
    #     "Programming Language :: Python",
    #     "Development Status :: 1 - Planning",
    #     "License :: OSI Approved",
    #     "Natural Language :: French",
    #     "Operating System :: OS Independent",
    #     "Programming Language :: Python :: 2.7",
    #     "Topic :: Communications",
    # ],


    # C'est un système de plugin, mais on s'en sert presque exclusivement
    # Pour créer des commandes, comme "django-admin".
    # La syntaxe est "nom-de-commande-a-creer = package.module:fonction".
    entry_points={
        'console_scripts': [
            'ph-echo = parapheur.core:echo',
            'ph-check = parapheur.core:check',
            'ph-init = parapheur.core:init',
            'ph-recupArchives = parapheur.core:recuparchives'
        ],
    },

    # A fournir uniquement si votre licence n'est pas listée dans "classifiers"
    # ce qui est notre cas
    license="GPL3",

)


# coding=utf-8
# module Parapheur
"""
CeCILL Copyright (c) 2006-2015, ADULLACT-projet
Initiated by ADULLACT-projet S.A.
Developped by ADULLACT-projet S.A.

contact@adullact-projet.coop

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
"""
import base64
import importlib
import os
import sys

import requests

import pprint

req_version = (3, 0)
cur_version = sys.version_info
isp3 = cur_version >= req_version

if isp3:
    # noinspection PyCompatibility
    from configparser import NoOptionError, NoSectionError
else:
    # noinspection PyCompatibility
    # noinspection PyCompatibility
    from ConfigParser import NoOptionError, NoSectionError

__author__ = 'lhameury'


# Handle login and curl requests
class Client:
    def runas(self, username):
        """
        Permet l'utilisation du client en tant qu'un utilisateur différent
        :param username: Nom d'utilisateur, celui-ci doit déjà être authentifié
        :return: Rien
        """
        if username not in self.__ticketlist:
            raise NotAuthenticatedError("Utilisateur {0} non authentifié !".format(username))
        elif username in self.__ticketlist and self.loggedAs != username:
            self.__ticket = self.__ticketlist[username]
            self.loggedAs = username
            pprint.header("Run as: '{0}'".format(username), False)

    def login(self, user, passw):
        """
        Récupération d'un ticket d'authentification pour l'utilisateur donné.
        Ce ticket est conservé en cas d'authentification avec un autre utilisateur.
        Il peut ensuite être ré-authentifié via la fonction 'runas'
        :param user: Nom d'utilisateur
        :param passw: Password
        :return: Rien
        """
        if user not in self.__ticketlist:
            pprint.info("Connexion   : {0} ".format(user), False, '')

            r = requests.post("{0}://{1}/alfresco/wcs/parapheur/api/login".format(self.__schema, self.server),
                              json=({'username': user, 'password': passw}), verify=False)

            if r.status_code == 200:
                json_response = r.json()
                # Le ticket est bien trouvé, on est connecté
                self.__ticket = json_response['data']['ticket']
                self.__ticketlist[user] = json_response['data']['ticket']
                self.islogged = True
                self.loggedAs = user
                pprint.success("OK".format(user), True)
            elif r.status_code == 403:
                json_response = r.json()
                pprint.error("KO - Erreur lors de la connexion: {0}".format(str(json_response['message'])), True)
            else:
                pprint.error("KO - Erreur inconnue :", True)
                print(r.text)
        else:
            self.runas(user)

    @staticmethod
    def load_class(full_class_string):
        """
        Chargement de classe automatique, à partir d'un chemin de classe
        Utilisé pour charger le module "API"
        :param full_class_string: Chaine de caractère représentant la classe à importer
        :return: La classe importée
        """
        class_data = full_class_string.split(".")
        module_path = ".".join(class_data[:-1])
        class_str = class_data[-1]

        module = importlib.import_module(module_path)
        # Finally, we retrieve the Class
        return getattr(module, class_str)

    @staticmethod
    def load_binary_file(filename):
        """
        Chargement d'un fichier binaire et conversion en Base64
        :param filename: Nom du fichier
        :return: Base64 du fichier
        """
        # use mode = "rb" to read binary file
        fin = open(filename, "rb")
        binary_data = fin.read()
        fin.close()
        # encode binary to base64 string (printable)
        return base64.b64encode(binary_data)

    def __getitem__(self, key):
        """
        Getter d'objet dans le dictionnaire
        """
        return self.__dict__[key]

    def __setitem__(self, key, value):
        """
        Setter d'objet dans le dictionnaire
        """
        self.__dict__[key] = value

    def __init__(self, config):
        """
        Fonction d'initialisation du module Parapheur.Client
        Authentification et mise en place des helpers d'API
        :param config:  Fichier de configuration,
                        doit comporter une section 'Parapheur' avec la valeur 'server' au minimum
        """
        pprint.log("\n[Parapheur-REST]", True)
        # Utilisateur log ?
        self.islogged = False
        try:
            # URL du serveur
            self.server = config.get("Parapheur", "server")
            pprint.info("Serveur     : " + self.server)
            # Ticket de connexion
            self.__ticketlist = {}
            self.__ticket = None
            self.loggedAs = None
            try:
                # Nom d'utilistaeur
                username = config.get("Parapheur", "username")
                pprint.info("Utilisateur : " + username)
                # Mot de passe utilisateur
                password = config.get("Parapheur", "password")
                # Autorité de certification - DISABLED
                # autority = config.get("Parapheur", "server_ac")
                # Récupération du chemin de l'AC
                # self.__autority = os.path.abspath(os.path.join(os.path.split(__file__)[0], os.pardir)) + '/' + autority
                self.__autority = ""
                # Set default schema
                try:
                    self.__schema = config.get("Parapheur", "schema")
                except NoOptionError:
                    self.__schema = "https"
                    pass
                # Login de l'utilisateur
                self.login(username, password)
            except NoOptionError:
                # Do nothing
                pass
        except (NoOptionError, NoSectionError) as e:
            pprint.error("Erreur fichier de configuration: {0}".format(str(e)), True)

    def __send(self, url, post, params, requesttype):
        """
        Envoi de requête vers le serveur d'API REST
        :param url: URL de requête
        :param post: Données POST
        :param params: Données GET
        :param requesttype: Type de requête (POST, GET, UPDATE, DELETE, ...)
        :return: Le résultat de la requête JSON, transformé en objet
        """
        # Handle parameters
        if not params:
            params = {}
        param_string = ""
        if isp3:
            for attr in params.keys():
                param_string += "&" + attr + "=" + params[attr]
        else:
            # noinspection PyCompatibility
            for attr in params.iterkeys():
                param_string += "&" + attr + "=" + params[attr]

        if post:
            post_data = post
        else:
            post_data = ()

        # Handle request
        r = requests.request(requesttype,
                             "{0}://{1}/alfresco/wcs{2}?ticket={3}{4}".format(self.__schema, self.server, url,
                                                                              self.__ticket, param_string),
                             json=post_data, verify=False)
        try:
            response = r.json()
        except:
            response = r.text
        return response

    def dopost(self, url, post=None, params=None):
        return self.__send(url, post, params, "POST")

    def doput(self, url, post=None, params=None):
        return self.__send(url, post, params, "PUT")

    def doget(self, url, params=None):
        return self.__send(url, None, params, "GET")

    def dodelete(self, url, params=None):
        return self.__send(url, None, params, "DELETE")

    def dodownload(self, url, filename):
        with open(filename, 'wb') as handle:
            response = requests.get(
                "{0}://{1}/alfresco/wcs{2}?ticket={3}".format(self.__schema, self.server, url, self.__ticket),
                stream=True, verify=False)

            if not response.ok:
                pprint.error("Error when downloading file", filename)

            for block in response.iter_content(chunk_size=1024):
                if block:  # filter out keep-alive new chunks
                    handle.write(block)

    def getspacenoderef(self, tenant=None):
        """
        Récupération du NodeRef du "workspace://SpacesStore", avec prise en compte du tenant si besoin
        :param tenant: Tenant sur lequel récupérer le node. Si vide, récupération sur le tenant ROOT
        :return: Le NodeRef du noeud
        """
        return self.doget(
            "/slingshot/doclib/treenode/node/alfresco/company/home/",
            {"children": "false",
             "libraryRoot": "alfresco://company/home",
             "max": "500"})['parent']['nodeRef']

    def executescript(self, scriptfile, *args, **kwargs):
        """
        Execution de script javascript via l'API REST

        :param scriptfile:  Fichier JS à exécuter
        :param kwargs:      runas: Execution en tant qu'utilisateur fourni
                            format: String à intégrer dans le script. Dans le fichier JS,
                                    remplacement des '{n}' (même principe que 'format')

        :return:            Résultat de la requête
        """
        runas = kwargs.get("runas", self.loggedAs)
        pprint.header("Exec: '{0}'".format(scriptfile, runas))
        with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "..", "scripts", "javascript",
                               scriptfile)) as script:
            scriptdata = script.read()
        return self.dopost("/de/fme/jsconsole/execute",
                           {"context": {},
                            "documentNodeRef": "",
                            "runas": kwargs.get("runas", self.loggedAs),
                            "script": scriptdata.format(*kwargs.get("format")) if kwargs.get("format") else scriptdata,
                            "spaceNodeRef": self.getspacenoderef(kwargs.get("tenant", "")),
                            "template": "",
                            "transaction": True,
                            "urlargs": ""})


class NotAuthenticatedError(Exception):
    """
    Custom exception pour problème d'authentification via l'API
    """
    pass



#!/usr/bin/env python
# coding=utf-8

# Récupération de l'objet 'parapheur' et de la configuration
import requests
# Parapheur API
import parapheur
# Scripts API
import scripts
# arguments
import gettext
from requests.packages.urllib3.exceptions import SubjectAltNameWarning
from requests.packages.urllib3.exceptions import InsecureRequestWarning

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
requests.packages.urllib3.disable_warnings(SubjectAltNameWarning)

__author__ = 'Lukas Hameury'

__all__ = ['init', 'echo', 'check']


def convertmessages(s):
    subdict = \
        {'positional arguments': 'Arguments',
         'optional arguments': 'Arguments',
         'show this help message and exit': 'Affiche ce message et quitte'}
    if s in subdict:
        s = subdict[s]
    return s
gettext.gettext = convertmessages
import argparse


def init():
    parser = argparse.ArgumentParser(
        prog='ph-init',
        description="Génère un fichier de configuration par défaut dans le répertoire courant")
    parser.add_argument('-p', help='Chemin du fichier de configuration')
    parser.add_argument('-c', help='Commande pour laquelle générer le fichier de configuration',
                        choices=["recuparchives"])

    args = parser.parse_args()

    filename = "script"
    path = "."
    if args.p:
        path = args.p
    if args.c:
        filename = args.c

    # Copy the configuration file
    parapheur.copyconfig(filename, path)

    print("Fichier de configuration 'iparapheur-utils.cfg' créé")


def echo():
    parser = argparse.ArgumentParser(
        prog='ph-echo',
        description="Lance un echo via webservice sur un iParapheur")

    parser.add_argument('-s', help="URL du serveur iParapheur")
    parser.add_argument('-c', help='Fichier de configuration')
    parser.add_argument('-u', help='Utilisateur')
    parser.add_argument('-p', help='Mot de passe')

    args = parser.parse_args()

    if args.c:
        parapheur.setconfig(args.c)
    if args.s:
        parapheur.setconfigproperty("server", args.s)
    if args.u:
        parapheur.setconfigproperty("username", args.u)
    if args.p:
        parapheur.setconfigproperty("password", args.p)

    # Initialisation d'API SOAP
    webservice = parapheur.getsoapclient()
    print(webservice.call().echo("coucou"))


def check():
    scripts.checkinstallation()


def recuparchives():
    parser = argparse.ArgumentParser(
        prog='ph-recupArchives',
        description="Lance une récupération / purge des archives")

    parser.add_argument('-s', help="URL du serveur iParapheur")
    parser.add_argument('-c', help='Fichier de configuration')
    parser.add_argument('-u', help='Utilisateur')
    parser.add_argument('-p', help='Mot de passe')

    parser.add_argument('-f', help='Répertoire de destination')
    parser.add_argument('-ps', help='Taille des pages à récupérer')
    parser.add_argument('-r', help='Chemins réduis des téléchargements', choices=["true", "false"])
    parser.add_argument('-pu', help='Active la purge les données', choices=["true", "false"])
    parser.add_argument('-d', help='Télécharge les données', choices=["true", "false"])
    parser.add_argument('-t', help='Filtre sur type')
    parser.add_argument('-st', help='Filtre sur sous-type')
    parser.add_argument('-w', help='Délai de conservation des données')

    args = parser.parse_args()

    if args.c:
        parapheur.setconfig(args.c)
    if args.s:
        parapheur.setconfigproperty("server", args.s)
    if args.u:
        parapheur.setconfigproperty("username", args.u)
    if args.p:
        parapheur.setconfigproperty("password", args.p)

    if args.f:
        parapheur.setconfigproperty("folder", args.f)
    if args.ps:
        parapheur.setconfigproperty("page_size", args.ps)
    if args.r:
        parapheur.setconfigproperty("use_reduced_download_path", args.r)
    if args.pu:
        parapheur.setconfigproperty("purge", args.pu)
    if args.d:
        parapheur.setconfigproperty("download", args.d)
    if args.t:
        parapheur.setconfigproperty("type_filter", args.t)
    if args.st:
        parapheur.setconfigproperty("subtype_filter", args.st)
    if args.w:
        parapheur.setconfigproperty("waiting_days", args.w)

    # Lancement du script
    scripts.recuparchives()

if __name__ == "__main__":
    recuparchives()
